
package landpjava;

import com.ibm.landp.spv.*;
import com.ibm.landp.shfile.*;
import com.ibm.landp.ej.*;
import com.ibm.landp.*;
import java.util.*;
import java.io.*;

/*
	LAN Distributed Platform Version 6 - LANDP

	This program is a simple demonstration of the use of the LANDP Java functionality
	to perform transactions on the Electronic Journal Server.
	A physical journal environment is used.
*/
public class AccessJournal implements LandpSPVConst, LandpSHFILEConst, LandpEJConst {

	/* Supervisor and remote request objects for communicating with LANDP */
	protected Supervisor spv;
	protected RmtReq req;

    	/* References to the Electronic Journal and underlying Shared File Server */
	protected SharedFileServer sfServer;
	protected ElectronicJournalServer ejServer;

	/* Client configuration. */
	protected String ejServName      = "ELECJO01";
	protected String sfServName      = "SHFILE01";
	protected String physicalJournalName  = "PHYSJO01";
	protected String recordFormat    = "EJREC   ";

	/* The current physical journal. */
	protected ElectronicJournal journal;

	/* User menu. */ 
	protected MenuPrompt optPrompt = new MenuPrompt( "What do you want to do?", new String[] { null, "List records", "Add record", "Delete record", null, "Exit program." } );

	public AccessJournal() {
		/* Use the Java Manager to assign our RmtReq it's own PID */
		req = new RmtReq(new Cprb(), RmtReq.MODE_OWN_PID);
		spv = new Supervisor(req);
	}

    	/*
		Run the client application.
		This approach guarantees correct initialisation and shut-down.
	*/
	public void runClient() {
		/* Connect to LANDP java manager and supervisor */
		ldpConnect();
		try {
			/* Create new instance of ElectronicJournalServer class */
			ejServer = new ElectronicJournalServer(ejServName, req);

			/* Create new instance of SharedFileServer class */
			sfServer = new SharedFileServer(req);

			/* Open Shared File Server for use in "online" mode */
			sfServer.start(sfServName, true);

			/*
				Loop until Electronic Journal is ready.
				It takes a few seconds for the shared file server to become active.
			*/
			System.out.print("Waiting for shared file server to come online ... ");
			while( !ejServer.getStatus() ) Thread.sleep(1000);
			System.out.println("Ok.");

			/* Allocate the physical journal if necessary. */
			allocateJournal();

			/* Perform operations on the current journal generation. */
			requestOptions(); 

		} catch( Exception e ) {
			/* Any unexpected exceptions are handled here. */
			e.printStackTrace();
			System.out.println("\nAn unexpected exception occurred.");
		}
		/* Disconnect from supervisor and unregister with java manager */
		ldpDisconnect();
	}

	/* Allocate the physical journal if it has not already been done. */
	public void allocateJournal() throws LandpEJException {
		journal = ejServer.getJournal( physicalJournalName );
		if( journal==null ) {
			System.out.println("Allocating physical journal ...");
			ejServer.openJournal( physicalJournalName );
			allocateJournal();
		}
	}

	/* List all the records in the physical journal. */
	public void listRecords() {
		try {
			ElectronicJournalRecord ejRec;

			/* Retrieve the first record. */
			ejRec = journal.searchRecord( "", LandpEJConst.LANDP_S_EJ_SEARCH_FIRST,
				0, recordFormat, false, LandpEJConst.LANDP_S_EJ_RECORD_FIRST );

			try {
				while( true ) {
					// Loop until an exception is thrown.
					System.out.println( ejRec.getData().substring(0,28) );
					ejRec = journal.searchRecord( "", LandpEJConst.LANDP_S_EJ_SEARCH_NEXT,
						0, recordFormat, false, LandpEJConst.LANDP_S_EJ_RECORD_MIDDLE );
				}
			} catch( LandpEJException e ) {
				if( e.getReturnCode() == 0x1005043 )
					/* PC error code: No more records. */
					System.out.println(" End of records.");
				else
					throw e;
			}

			/* Search for the first record to end the transaction. */
			ejRec = journal.searchRecord( "", LandpEJConst.LANDP_S_EJ_SEARCH_FIRST,
				0, recordFormat, false, LandpEJConst.LANDP_S_EJ_RECORD_LAST );

		} catch( LandpEJException e ) {
			System.out.println("Landp error code : "+e.getReturnCode() );
		}
	}

	/* Add the specified financial record to the journal. */
	public void addRecord( String name ) {
		/* Truncate name if necessary */
		if( name.length() > 20 ) name = name.substring(0,20);
		/* Pad name with spaces if necessary. */
		if( name.length() < 20 ) {
			StringBuffer sb = new StringBuffer( name );
			for( int n=name.length(); n<20 ; n++ ) sb.append(" ");
			name = sb.toString();
		}

		try {
			ElectronicJournalRecord ejRec = new ElectronicJournalRecord( name+"00000000" );
			journal.addRecord( ejRec, recordFormat, LandpEJConst.LANDP_S_EJ_RECORD_ONLY );
		} catch( LandpEJException e ) {
			System.out.println( e.getMessage() );
			System.out.println("Landp error code : "+e.getReturnCode() );
		}
	}

	/* Delete the record whose customer name field matches the one specified. */
	public void deleteRecord( String customerName ) {
		try {
			/* Retrieve the record before deleting it. This is only used for the transaction flags. */
			journal.searchRecord( "CUSTNAME = '"+customerName+"'", LandpEJConst.LANDP_S_EJ_SEARCH_FIRST,
				0, recordFormat, false, LandpEJConst.LANDP_S_EJ_RECORD_FIRST );
			journal.deleteRecord( "CUSTNAME = '"+customerName+"'", LandpEJConst.LANDP_S_EJ_SEARCH_FIRST,
				0, recordFormat, LandpEJConst.LANDP_S_EJ_RECORD_LAST );
		} catch( LandpEJException e ) {
			System.out.println( e.getMessage() );
			System.out.println( "Landp error code : "+e.getReturnCode() );
		}
	}

	/* Loop to prompt the user for record operations on the current journal generation. */
	public void requestOptions() {
		boolean condition = true;
		while( condition ) {
			switch( optPrompt.showMenu() ) {
				case 1:
					listRecords();
					break;
				case 2:
					addRecord( optPrompt.readLine("Please enter customer name > ") );
					break;
				case 3:
					deleteRecord( optPrompt.readLine("Please enter customer name > ") );
					break;
				case 5:
					condition = false;
					break;
			}
		}
	}

	/* Register with LANDP. */
	public void ldpConnect() {
		System.out.println("Registering with LANDP Java Manager and connecting to Supervisor.");
		try {
			req.register();
			spv.connect();
		} catch (Exception e) {
			/* There was a problem registering */
			e.printStackTrace();
			System.out.println("\nException occurred while connecting with LANDP!");
			System.exit(-1);
		}
	}

	/* Unregister with LANDP */
	public void ldpDisconnect() {
		System.out.println("Disconnecting from LANDP.");
		try {
    			spv.disconnect();
			req.unregister();
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("\nException occurred while disconnecting from LANDP!");
			System.exit(-1);
		}
	}

	public static void main( String[] args ) {
	    new AccessJournal().runClient();
	}
}

/* An object of this class provides command line input. */
class MenuPrompt {
	protected String title;
	protected Vector menuOptions = new Vector();
	protected BufferedReader inputReader = new BufferedReader(new InputStreamReader(System.in));

	/* Construct a new MenuPrompt with the specified options. */
	public MenuPrompt( String title, String[] options ) {
		this.title = title;
		if( options.length < 1 ) throw new IllegalArgumentException("You must have at least one menu option!");
		for( int n=0; n<options.length; n++ ) menuOptions.add( options[n] );
 	}

	/* A yes/no prompt. */
	public boolean yesNo( String question ) {
		String s = "x";
		boolean result=false, cond=true;
		while( cond ) {
			System.out.print( question + " (y/n) > " );
			try { s = inputReader.readLine().toLowerCase(); } catch( Exception e ) {}
			if( s.equals("y") ) {
				result = true;
				cond = false;
			}
			if( s.equals("n") ) {
				result = false;
				cond = false;
			}

		}
		return result;
	}

	/* Get a line of text from the command line */
	public String readLine( String prompt ) {
		String s = null;
		System.out.print(prompt);
		try { s = inputReader.readLine(); } catch( Exception e ) {}
		return s;
	}

	/* Allow the user to make a selection. Returns the value the user typed or -1. */
	public int showMenu() {
		System.out.println(title);
		for( int n=0; n<menuOptions.size(); n++ ) {
			String value = (String)menuOptions.elementAt(n);
			if( value != null ) System.out.println( " "+ n + ") " + value );
		}
		System.out.print("Please enter a number > ");
		try {
			return Integer.parseInt( inputReader.readLine() );
		} catch( Exception e ) {
			return -1;
		}
	}
}



